const BaseStepLogger                   = require('../StepLogger');
const RedisPubDecorator                = require('./redisPubDecorator');
const RedisLogger                      = require('./RedisLogger');
const RedisTaskLogger                  = require('./TaskLogger');
const { STATUS }                       = require('../enums');

class RedisStepLogger extends BaseStepLogger {
    constructor(step, opts) {
        super(step, opts);
        const extendOpts = Object.assign({}, step, opts);
        extendOpts.key = `${step.accountId}:${step.jobId}:${step.name}`;
        const redisConnection = RedisTaskLogger.getRedisConnectionFromCache(extendOpts);
        this.writter = new RedisPubDecorator(extendOpts, new RedisLogger(redisConnection, extendOpts));
        this.writter.setStrategies(`${step.accountId}:${step.jobId}`);
    }

    async restore() {
        this.status = await this.writter.child('status').get();
        this.pendingApproval = this.status === STATUS.PENDING_APPROVAL;
    }

    _reportLog(message) {
        this.writter.child('logs').push(message);
    }

    _reportLastUpdate() {
        this.writter.child('lastUpdate').set(this.lastUpdate);
    }

    _reportPrevioulyExecuted() {
        this.writter.child('previouslyExecuted').set(this.previouslyExecuted);
    }

    _reportStatus() {
        this.writter.child('status').set(this.status);
    }

    _reportFinishTimestamp() {
        this.writter.child('finishTimeStamp').set(this.finishTimeStamp);
    }

    _reportCreationTimestamp() {
        this.writter.child('creationTimeStamp').set(this.creationTimeStamp);
    }

    _reportMemoryUsage(time, memoryUsage) {
        this.writter.child('metrics').child('memory').push({ time, usage: memoryUsage });
    }

    _reportCpuUsage(time, cpuUsage) {
        this.writter.child('metrics').child('cpu').push({ time, usage: cpuUsage });
    }

    _reportLogSize(size) {
        this.writter.child('metrics').child('logs').child('total').set(size);
    }

    reportName() {
        this.writter.child('name').set(this.name);
    }

    clearLogs() {
        this.writter.child('logs').set({});
    }

    async delete() {
        return this.writter.remove();
    }
}

module.exports = RedisStepLogger;
